package edu.lawrence.graphpad

import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.ui.geometry.Offset
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class GraphModel(private val dao : GraphDao) : ViewModel() {
    var graphs = mutableStateOf<List<GraphInfo>>(listOf<GraphInfo>())
    var curGraph = mutableIntStateOf(0)
    var vertices = mutableStateOf<List<Vertex>>(listOf<Vertex>())
    var edges = mutableStateOf<List<Edge>>(listOf<Edge>())
    var dragging = mutableStateOf<Vertex?>(null)
    var start = mutableStateOf<Vertex?>(null)

    init {
        viewModelScope.launch {
            var graphList: List<GraphInfo>
            withContext(Dispatchers.IO) {
                graphList = dao.loadGraphs()
            }
            withContext(Dispatchers.Main) {
                graphs.value = graphList
            }
        }
    }

    fun setGraph(id : Int) {
        curGraph.intValue = id
        viewModelScope.launch {
            var vertexList : List<Vertex>
            withContext(Dispatchers.IO) {
                vertexList = dao.loadVertices(id)
            }
            withContext(Dispatchers.Main) {
                vertices.value = vertexList
            }
            var edgeInfoList : List<EdgeInfo>
            val edgeList = mutableListOf<Edge>()
            withContext(Dispatchers.IO) {
                edgeInfoList = dao.loadEdges(id)
                for(ei in edgeInfoList) {
                    val o = dao.findVertex(ei.origin)
                    val d = dao.findVertex(ei.dest)
                    edgeList.add(Edge(origin=o,dest=d))
                }
            }
            withContext(Dispatchers.Main) {
                edges.value = edgeList.toList()
            }
        }
    }

    fun makeGraph(name : String) {
        val gi = GraphInfo(name = name)
        viewModelScope.launch {
            var graphList: List<GraphInfo>
            withContext(Dispatchers.IO) {
                dao.addGraph(gi)
                graphList = dao.loadGraphs()
            }
            withContext(Dispatchers.Main) {
                graphs.value = graphList
            }
        }
    }

    fun makeVertex(center : Offset) {
        var v = Vertex(centerX = center.x,centerY = center.y,graph = curGraph.intValue)
        var id : Long
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                id = dao.addVertex(v)
                v = dao.findVertex(id.toInt())
            }
            withContext(Dispatchers.Main) {
                vertices.value += v
            }
        }
    }

    fun startDrag(pt:Offset) {
        for (v in vertices.value) {
            if(v.containsPoint(pt)) {
                dragging.value = v
            }
        }
    }

    fun continueDrag(pt:Offset) {
        if(dragging.value != null) {
            val v = dragging.value!!
            v.centerX = pt.x
            v.centerY = pt.y
        }
    }

    fun endDrag(pt:Offset) {
        if(dragging.value != null) {
            val v = dragging.value!!
            viewModelScope.launch {
                withContext(Dispatchers.IO) {
                    dao.updateVertex(v)
                }
            }
            dragging.value = null
        }
    }

    fun startDraw(pt:Offset) {
        for (v in vertices.value) {
            if(v.containsPoint(pt)) {
                start.value = v
            }
        }
    }

    fun endDraw(pt:Offset) {
        if(start.value != null) {
            val s = start.value!!
            for (v in vertices.value) {
                if(v.containsPoint(pt) && v != s) {
                    val newEdge = Edge(origin = s,dest = v)
                    edges.value += newEdge
                    val ei = EdgeInfo(origin = s.id,dest=v.id,graph=curGraph.intValue)
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            dao.addEdge(ei)
                        }
                    }
                    break
                }
            }
        }
        start.value = null
    }
}