package edu.lawrence.graphpad

import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.Canvas
import androidx.compose.foundation.border
import androidx.compose.foundation.clickable
import androidx.compose.foundation.gestures.detectDragGestures
import androidx.compose.foundation.gestures.detectTapGestures
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.Button
import androidx.compose.material3.Card
import androidx.compose.material3.Switch
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TextField
import androidx.compose.material3.adaptive.ExperimentalMaterial3AdaptiveApi
import androidx.compose.material3.adaptive.layout.ListDetailPaneScaffoldRole
import androidx.compose.material3.adaptive.navigation.NavigableListDetailPaneScaffold
import androidx.compose.material3.adaptive.navigation.ThreePaneScaffoldNavigator
import androidx.compose.material3.adaptive.navigation.rememberListDetailPaneScaffoldNavigator
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.pointer.pointerInput
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.window.Dialog
import androidx.room.Room
import edu.lawrence.graphpad.ui.theme.GraphPadTheme

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val db = Room.databaseBuilder(
            application,
            GraphDatabase::class.java, "grades"
        ).build()
        val graphs = GraphModel(dao = db.graphDao())
        enableEdgeToEdge()
        setContent {
            GraphPadTheme {
                CanvasApp(gm = graphs)
            }
        }
    }
}

@Composable
@OptIn(ExperimentalMaterial3AdaptiveApi::class)
fun CanvasApp(gm : GraphModel) {
    val navigator = rememberListDetailPaneScaffoldNavigator<Any>()
    NavigableListDetailPaneScaffold(
        navigator = navigator,
        listPane = { GraphList(gm, onClick = { gi -> navigator.navigateTo(
            ListDetailPaneScaffoldRole.Detail, content = gi) }) },
        detailPane = { GraphDetailPane(gm,navigator) }
    )
}

@Composable
fun GraphList(gm : GraphModel,onClick : (GraphInfo)->Unit) {
    val showDialog = remember { mutableStateOf(false) }
    Column(verticalArrangement = Arrangement.Center, modifier = Modifier.padding(20.dp)) {
        gm.graphs.value.forEach { gi ->
            Text(gi.name,
                modifier = Modifier.padding(10.dp).clickable(onClick = { onClick(gi) }))
        }
        Button(onClick = { showDialog.value = true }) {
            Text("Make New Graph")
        }
    }
    if(showDialog.value) {
        NameDialog(
            onConfirm = {
                gm.makeGraph(it)
                showDialog.value = false
            },
            onDismiss = { showDialog.value = false })
    }
}

@Composable
fun NameDialog(onConfirm: (String) -> Unit,onDismiss : () -> Unit) {
    val name = remember { mutableStateOf("") }

    Dialog(onDismissRequest = { onDismiss() }) {
        Card(
            modifier = Modifier.padding(16.dp),
            shape = RoundedCornerShape(16.dp),
        ) {
            Column(
                modifier = Modifier,
                verticalArrangement = Arrangement.Center
            ) {
                Text(
                    text = "Enter a name for a new graph",
                    modifier = Modifier.padding(16.dp),
                )
                TextField(value = name.value,
                    onValueChange = { name.value = it },
                    modifier = Modifier.fillMaxWidth())
            }
            Row(
                modifier = Modifier
                    .fillMaxWidth(),
                horizontalArrangement = Arrangement.Center,
            ) {
                TextButton(
                    onClick = { onDismiss() },
                    modifier = Modifier.padding(8.dp),
                ) {
                    Text("Dismiss")
                }
                TextButton(
                    onClick = { onConfirm(name.value) },
                    modifier = Modifier.padding(8.dp),
                ) {
                    Text("Confirm")
                }
            }
        }
    }
}

@Composable
@OptIn(ExperimentalMaterial3AdaptiveApi::class)
private fun GraphDetailPane(gm : GraphModel,navigator: ThreePaneScaffoldNavigator<Any>) {
    val graph : GraphInfo? = navigator.currentDestination?.content as GraphInfo?
    if(graph != null) {
        gm.setGraph(graph.id)
        CanvasDemo(gm)
    } else {
        Text("Make a selection.")
    }
}

@Composable
fun CanvasDemo(graph : GraphModel,modifier: Modifier = Modifier) {
    val vertices = remember { graph.vertices }
    val drawingEdge = remember { mutableStateOf(false)}
    val startOffset = remember { mutableStateOf(Offset.Zero) }
    val curOffset = remember { mutableStateOf(Offset.Zero) }

    fun startDrag(offset: Offset) {
        startOffset.value = offset
        curOffset.value = offset
        if(drawingEdge.value) {
            graph.startDraw(offset)
        } else {
            graph.startDrag(offset)
        }
    }

    fun continueDrag(dragAmount:Offset) {
        curOffset.value = Offset(x = curOffset.value.x + dragAmount.x,y = curOffset.value.y + dragAmount.y)
        if(!drawingEdge.value) {
            graph.continueDrag(curOffset.value)
        }
    }

    fun endDrag() {
        if(drawingEdge.value) {
            graph.endDraw(curOffset.value)
        } else {
            graph.endDrag(curOffset.value)
        }
        startOffset.value = Offset.Zero
        curOffset.value = Offset.Zero
    }

    Column(verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally, modifier = modifier) {
        Spacer(modifier = Modifier.height(20.dp))
        Row(
            horizontalArrangement = Arrangement.Start,
            verticalAlignment = Alignment.CenterVertically
        ) {
            Text("Draw Edges:")
            Spacer(modifier = Modifier.width(10.dp))
            Switch(
                checked = drawingEdge.value,
                onCheckedChange = { drawingEdge.value = it })
        }
        Canvas(modifier = Modifier.fillMaxWidth().height(400.dp).padding(10.dp).border(BorderStroke(2.dp, Color.Blue))
            .pointerInput(Unit) {
                detectTapGestures(onPress = { startDrag(it) })
            }
            .pointerInput(Unit){
                detectDragGestures(onDrag = {_,amount->continueDrag(amount)},
                    onDragEnd = {endDrag()})
            }) {
            for( edge in graph.edges.value) {
                edge.drawIn(this)
            }
            for( vertex in graph.vertices.value) {
                vertex.drawIn(this)
            }
            if(startOffset.value != curOffset.value && drawingEdge.value) {
                drawLine(
                    color = Color.Black,
                    start = startOffset.value,
                    end = curOffset.value,
                    strokeWidth = 1.0f
                )
            }
        }
        Button(onClick = { graph.makeVertex(Offset(x = 600f,y = 600f)) }) {
            Text("Make New Vertex")
        }
    }
}
